__title__ = 'cobald'
__summary__ = 'COBalD - the Opportunistic Balancing Deamon'
__url__ = 'https://github.com/MaineKuehn/cobald'

__version__ = '0.0.1'
__author__ = 'Eileen Kuehn, Max Fischer'
__email__ = 'maxfischer2781@gmail.com'
__copyright__ = '2018 %s' % __author__
__keywords__ = 'opportunistic scheduling scheduler demand feedback loop asyncio'
